/*
 * This file is part of libdom.
 * Licensed under the MIT License,
 *                http://www.opensource.org/licenses/mit-license.php
 * Copyright 2009 Bo Yang <struggleyb.nku@gmail.com>
 */

#ifndef dom_core_comment_h_
#define dom_core_comment_h_

#include <stdbool.h>

#include <dom/core/exceptions.h>
#include <dom/core/characterdata.h>

typedef struct dom_comment dom_comment;

#endif
