/*
 * This file is part of libdom.
 * Licensed under the MIT License,
 *                http://www.opensource.org/licenses/mit-license.php
 * Copyright 2007 John-Mark Bell <jmb@netsurf-browser.org>
 */

#ifndef dom_core_processinginstruction_h_
#define dom_core_processinginstruction_h_

typedef struct dom_processing_instruction dom_processing_instruction;

#endif
